#include "Builder/Builders/BonusBuilder.h"
#include "MazeContent/Buildings/Wall.h"
#include "MazeContent/Buildings/Pass.h"
#include "MazeContent/Buildings/Teleport.h"

BonusBuider::BonusBuider(float width, float height, float room_size):
        CommonBuilder(width, height, room_size) {}



void BonusBuider::create_rooms() {
    size_t rows = static_cast<int>(m_height / m_room_size) % 2 == 0 ? m_height / m_room_size - 1 : m_height / m_room_size;
    size_t cols = static_cast<int>(m_width / m_room_size) % 2 == 0 ? m_width / m_room_size - 1 : m_width / m_room_size;

    sf::Vector2f top_left_origin_pos{ (m_width - cols * m_room_size) / 2 + m_room_size / 2,
                                      (m_height - rows * m_room_size) / 2 + m_room_size / 2 };

    for (size_t i = 0; i < rows; ++i) {
        std::vector<std::unique_ptr<Room>> row;
        for (size_t j = 0; j < cols; ++j) {
            if (i % 2 == 1 || j % 2 == 1) {
                auto room = std::make_unique<Room>(m_room_size);
                room->set_position(sf::Vector2f{ top_left_origin_pos.x + j * m_room_size,
                                                 top_left_origin_pos.y + i * m_room_size });
                row.emplace_back(std::move(room));
            }
        }
        m_rooms.emplace_back(std::move(row));
    }
}

static const std::vector<std::pair<int, int>> excluded_rooms_l_r = {
        {0, 2}, {0, 7}, {12, 2}, {12, 7}
};

static const std::vector<std::pair<int, int>> excluded_rooms_u_d = {
        {6, 0}, {6, 4}, {6, 5}, {6, 9}
};

static bool is_excluded(int i, int j) {
    for (const auto& excluded : excluded_rooms_l_r) {
        if (excluded.first == i && excluded.second == j)
            return true;
    }
    for (const auto& excluded : excluded_rooms_u_d) {
        if (excluded.first == i && excluded.second == j)
            return true;
    }
    return false;
}

static void set_rooms_teleport(BonusBuider& BD, std::shared_ptr<Teleport> teleport) {
    for (const auto &excluded: excluded_rooms_l_r) {
        size_t i = excluded.first;
        size_t j = excluded.second;
        auto left_wall_1 = std::make_shared<Wall>(*(BD.m_rooms[i][j]));
        auto right_wall_1 = std::make_shared<Wall>(*(BD.m_rooms[i][j]));
        BD.m_rooms[i][j]->set_side(Room::LEFT, left_wall_1);
        BD.m_rooms[i][j]->set_side(Room::RIGHT, right_wall_1);
        if (i == 0) {
            BD.m_rooms[i][j]->set_side(Room::UP, teleport);
            auto my_down_pass = std::make_shared<Pass>(*(BD.m_rooms[i][j]), *(BD.m_rooms[i + 1][2 * j + 1]));
            auto neighbour_down_pass = std::make_shared<Pass>(*(BD.m_rooms[i + 1][2 * j + 1]), *(BD.m_rooms[i + 2][j]));
            BD.m_rooms[i][j]->set_side(Room::DOWN, my_down_pass);
            BD.m_rooms[i + 1][2 * j + 1]->set_side(Room::UP, my_down_pass);
            BD.m_rooms[i + 1][2 * j + 1]->set_side(Room::DOWN, neighbour_down_pass);
            BD.m_rooms[i + 2][j]->set_side(Room::UP, neighbour_down_pass);
        } else if (i == 12) {
            BD.m_rooms[i][j]->set_side(Room::DOWN, teleport);
            auto my_up_pass = std::make_shared<Pass>(*(BD.m_rooms[i][j]), *(BD.m_rooms[i - 1][2 * j + 1]));
            auto neighbour_up_pass = std::make_shared<Pass>(*(BD.m_rooms[i - 1][2 * j + 1]), *(BD.m_rooms[i - 2][j]));
            BD.m_rooms[i][j]->set_side(Room::UP, my_up_pass);
            BD.m_rooms[i - 1][2 * j + 1]->set_side(Room::DOWN, my_up_pass);
            BD.m_rooms[i - 1][2 * j + 1]->set_side(Room::UP, neighbour_up_pass);
            BD.m_rooms[i - 2][j]->set_side(Room::DOWN, neighbour_up_pass);
        }
    }
    for (const auto &excluded: excluded_rooms_u_d) {
        size_t i = excluded.first;
        size_t j = excluded.second;
        if (j == 0 || j == 5) {
            BD.m_rooms[i][j]->set_side(Room::LEFT, teleport);
            auto right_wall = std::make_shared<Wall>(*(BD.m_rooms[i][j]));
            BD.m_rooms[i][j]->set_side(Room::RIGHT, right_wall);
        } if (j == 4 || j == 9) {
            BD.m_rooms[i][j]->set_side(Room::RIGHT, teleport);
            auto left_wall = std::make_shared<Wall>(*(BD.m_rooms[i][j]));
            BD.m_rooms[i][j]->set_side(Room::LEFT, left_wall);
        }
    }
    BD.m_rooms[0][2]->get_side(Room::UP)->prepare_for_drawing();
}

void BonusBuider::set_rooms_sides() {
    auto teleport = std::make_shared<Teleport>(*m_rooms[0][2], *m_rooms[0][7], *m_rooms[6][0], *m_rooms[6][4],
                                               *m_rooms[6][5], *m_rooms[6][9], *m_rooms[12][2], *m_rooms[12][7]);
    set_rooms_teleport(*this, teleport);
    for (size_t i = 0; i < m_rooms.size(); ++i) {
        for (size_t j = 0; j < m_rooms.at(i).size(); ++j) {
            if (is_excluded(i, j))
                continue;
            if (i % 2 == 0) {
                auto left_wall_1 = std::make_shared<Wall>(*m_rooms[i][j]);
                auto right_wall_1 = std::make_shared<Wall>(*m_rooms[i][j]);
                m_rooms[i][j]->set_side(Room::LEFT, left_wall_1);
                m_rooms[i][j]->get_side(Room::LEFT)->prepare_for_drawing();
                m_rooms[i][j]->set_side(Room::RIGHT, right_wall_1);
                m_rooms[i][j]->get_side(Room::RIGHT)->prepare_for_drawing();
                if (i == 0) {
                    auto up_wall_1 = std::make_shared<Wall>(*m_rooms[i][j]);
                    auto my_down_pass = std::make_shared<Pass>(*m_rooms[i][j], *m_rooms[i + 1][2 * j + 1]);
                    auto neighbour_down_pass = std::make_shared<Pass>(*m_rooms[i + 1][2 * j + 1], *m_rooms[i + 2][j]);
                    m_rooms[i][j]->set_side(Room::UP, up_wall_1);
                    m_rooms[i][j]->get_side(Room::UP)->prepare_for_drawing();
                    m_rooms[i][j]->set_side(Room::DOWN, my_down_pass);
                    m_rooms[i][j]->get_side(Room::DOWN)->prepare_for_drawing();
                    m_rooms[i + 1][2 * j + 1]->set_side(Room::UP, my_down_pass);
                    m_rooms[i + 1][2 * j + 1]->get_side(Room::UP)->prepare_for_drawing();
                    m_rooms[i + 1][2 * j + 1]->set_side(Room::DOWN, neighbour_down_pass);
                    m_rooms[i + 1][2 * j + 1]->get_side(Room::DOWN)->prepare_for_drawing();
                    m_rooms[i + 2][j]->set_side(Room::UP, neighbour_down_pass);
                    m_rooms[i + 2][j]->get_side(Room::UP)->prepare_for_drawing();
                } else if (i == m_rooms.size() - 1) {
                    auto down_wall_1 = std::make_shared<Wall>(*m_rooms[i][j]);
                    auto my_up_pass = std::make_shared<Pass>(*m_rooms[i][j], *m_rooms[i - 1][2 * j + 1]);
                    auto neighbour_up_pass = std::make_shared<Pass>(*m_rooms[i - 1][2 * j + 1], *m_rooms[i - 2][j]);
                    m_rooms[i][j]->set_side(Room::DOWN, down_wall_1);
                    m_rooms[i][j]->get_side(Room::DOWN)->prepare_for_drawing();
                    m_rooms[i][j]->set_side(Room::UP, my_up_pass);
                    m_rooms[i][j]->get_side(Room::UP)->prepare_for_drawing();
                    m_rooms[i - 1][2 * j + 1]->set_side(Room::DOWN, my_up_pass);
                    m_rooms[i - 1][2 * j + 1]->get_side(Room::DOWN)->prepare_for_drawing();
                    m_rooms[i - 1][2 * j + 1]->set_side(Room::UP, neighbour_up_pass);
                    m_rooms[i - 1][2 * j + 1]->get_side(Room::UP)->prepare_for_drawing();
                    m_rooms[i - 2][j]->set_side(Room::DOWN, neighbour_up_pass);
                    m_rooms[i - 2][j]->get_side(Room::DOWN)->prepare_for_drawing();
                }
            }
            else if (j % 2 == 0) {
                auto up_wall_2 = std::make_shared<Wall>(*m_rooms[i][j]);
                auto down_wall_2 = std::make_shared<Wall>(*m_rooms[i][j]);
                m_rooms[i][j]->set_side(Room::UP, up_wall_2);
                m_rooms[i][j]->get_side(Room::UP)->prepare_for_drawing();
                m_rooms[i][j]->set_side(Room::DOWN, down_wall_2);
                m_rooms[i][j]->get_side(Room::DOWN)->prepare_for_drawing();
                if (j == 0) {
                    auto left_wall_2 = std::make_shared<Wall>(*m_rooms[i][j]);
                    m_rooms[i][j]->set_side(Room::LEFT, left_wall_2);
                    m_rooms[i][j]->get_side(Room::LEFT)->prepare_for_drawing();
                }
                else if (j == m_rooms.at(i).size() - 1) {
                    auto right_wall_2 = std::make_shared<Wall>(*m_rooms[i][j]);
                    m_rooms[i][j]->set_side(Room::RIGHT, right_wall_2);
                    m_rooms[i][j]->get_side(Room::RIGHT)->prepare_for_drawing();
                }
            }
            else if (j % 2 == 1) {
                auto left_pass = std::make_shared<Pass>(*m_rooms[i][j], *m_rooms[i][j - 1]);
                auto right_pass = std::make_shared<Pass>(*m_rooms[i][j], *m_rooms[i][j + 1]);
                m_rooms[i][j]->set_side(Room::LEFT, left_pass);
                m_rooms[i][j]->get_side(Room::LEFT)->prepare_for_drawing();
                m_rooms[i][j - 1]->set_side(Room::RIGHT, left_pass);
                m_rooms[i][j - 1]->get_side(Room::RIGHT)->prepare_for_drawing();
                m_rooms[i][j]->set_side(Room::RIGHT, right_pass);
                m_rooms[i][j]->get_side(Room::RIGHT)->prepare_for_drawing();
                m_rooms[i][j + 1]->set_side(Room::LEFT, right_pass);
                m_rooms[i][j + 1]->get_side(Room::LEFT)->prepare_for_drawing();
                if (!(i == 1 || i == m_rooms.size() - 2)) {
                    auto up_pass = std::make_shared<Pass>(*m_rooms[i][j], *m_rooms[i - 1][(j -1) / 2]);
                    auto down_pass = std::make_shared<Pass>(*m_rooms[i][j], *m_rooms[i + 1][(j -1) / 2]);
                    m_rooms[i][j]->set_side(Room::UP, up_pass);
                    m_rooms[i][j]->get_side(Room::UP)->prepare_for_drawing();
                    m_rooms[i - 1][(j -1) / 2]->set_side(Room::DOWN, up_pass);
                    m_rooms[i - 1][(j -1) / 2]->get_side(Room::DOWN)->prepare_for_drawing();
                    m_rooms[i][j]->set_side(Room::DOWN, down_pass);
                    m_rooms[i][j]->get_side(Room::DOWN)->prepare_for_drawing();
                    m_rooms[i + 1][(j -1) / 2]->set_side(Room::UP, down_pass);
                    m_rooms[i + 1][(j -1) / 2]->get_side(Room::UP)->prepare_for_drawing();
                }
            }
        }
    }
}