#include "MazeContent/Entities/Pacman.h"
#include <Config.h>
#include "Events/DeleteStaticEntity.hpp"
#include "Events/LostGame.hpp"
#include "MazeContent/Entities/StaticEntities/Food.h"
#include "MazeContent/Entities/DynamicEntities/Enemy.h"

Pacman::Pacman() {
    set_texture(&PacmanTexture::Instance());
}

void Pacman::set_texture(Texture* ptr_texture) {
    m_ptr_texture = ptr_texture;
}

void Pacman::prepare_for_drawing() {
    m_sprite.setTexture(m_ptr_texture->get_texture());
    if (m_direction == config::RIGHT) {
        m_sprite.setTextureRect(sf::IntRect(0, 0, m_ptr_texture->get_texture().getSize().x / m_rect_count,
                                            (m_ptr_texture->get_texture().getSize().y) / 4));
    } else if (m_direction == config::DOWN) {
        m_sprite.setTextureRect(sf::IntRect(0, (m_ptr_texture->get_texture().getSize().y) / (4),
                             m_ptr_texture->get_texture().getSize().x / m_rect_count, (m_ptr_texture->get_texture().getSize().y) / (4)));
    } else if (m_direction == config::LEFT) {
        m_sprite.setTextureRect(sf::IntRect(0, (m_ptr_texture->get_texture().getSize().y) / (m_rect_count),
                             m_ptr_texture->get_texture().getSize().x / m_rect_count, (m_ptr_texture->get_texture().getSize().y) / 4 ));
    } else if (m_direction == config::UP) {
        m_sprite.setTextureRect(sf::IntRect(0, 3 * (m_ptr_texture->get_texture().getSize().y) / (2 * m_rect_count),
                             m_ptr_texture->get_texture().getSize().x / m_rect_count, (m_ptr_texture->get_texture().getSize().y)));
    }
    m_sprite.setPosition(m_ptr_room->get_position() + sf::Vector2f(-27, -27));
    m_sprite.setScale(0.15, 0.15);
}

void Pacman::next_animation() {
    sf::IntRect rect = m_sprite.getTextureRect();
    rect.left = ((rect.left / (rect.width) + 1) % m_rect_count) * rect.width;
    m_sprite.setTextureRect(rect);
}

void Pacman::draw_into(sf::RenderWindow& window) const {
    window.draw(m_sprite);
}

void Pacman::move(Room::Direction direction) {
    m_ptr_room->get_side(direction)->enter(this);
}

std::unique_ptr<IGameEvent> Pacman::visit(Food* ptr_food) {
    if (ptr_food->get_location() != this->get_location()) {
        return {};
    }
    return std::make_unique<DeleteStaticEntity>(ptr_food);
}

std::unique_ptr<IGameEvent> Pacman::visit(Enemy* ptr_enemy) {
    if (ptr_enemy->get_location() != this->get_location()) {
        return {};
    }
    return std::make_unique<LostGame>();
}